using FishFactoryContracts.BindingModels;
using FishFactoryContracts.BusinessLogicsContracts;
using FishFactoryDataModels.Enums;
using Microsoft.Extensions.Logging;
using FishFactoryContracts.DI;

namespace FishFactoryView
{
	public partial class MainForm : Form
	{
		private readonly ILogger _logger;
		private readonly IOrderLogic _orderLogic;
		private readonly IReportLogic _reportLogic;
		private readonly IWorkProcess _workProcess;
		private readonly IBackUpLogic _backUpLogic;
		public MainForm(ILogger<MainForm> logger, IOrderLogic orderLogic, IReportLogic reportLogic, IWorkProcess workProcess, IBackUpLogic backUpLogic)
		{
			InitializeComponent();
			_logger = logger;
			_orderLogic = orderLogic;
			_reportLogic = reportLogic;
			_workProcess = workProcess;
			_backUpLogic = backUpLogic;
		}
		private void FormMain_Load(object sender, EventArgs e)
		{
			LoadData();
		}
		private void LoadData()
		{
			_logger.LogInformation(" ");
			try
			{
				dataGridView.FillAndConfigGrid(_orderLogic.ReadList(null));
				_logger.LogInformation(" ");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "  ");
				MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		private void ComponentToolStripMenuItem_Click(object sender, EventArgs
	   e)
		{
			var service = DependencyManager.Instance.Resolve<FormComponents>();
			if (service is FormComponents form)
			{
				form.ShowDialog();
			}
		}
		private void CannedToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormCanneds>();
			if (service is FormCanneds form)
			{
				form.ShowDialog();
			}
		}
		private void ButtonCreateOrder_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormCreateOrder>();
			if (service is FormCreateOrder form)
			{
				form.ShowDialog();
				LoadData();
			}
		}
		private void ButtonIssuedOrder_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id =
			   Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation(" {id}.    ''", id);
				try
				{
					var operationResult = _orderLogic.DeliveryOrder(CreateBindingModel(id));
					if (!operationResult)
					{
						throw new Exception("  .    .");
					}
					_logger.LogInformation(" {id} ", id);
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "    ");
					MessageBox.Show(ex.Message, "", MessageBoxButtons.OK,
				   MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonRef_Click(object sender, EventArgs e)
		{
			LoadData();
		}
		private OrderBindingModel CreateBindingModel(int id, bool isDone = false)
		{
			return new OrderBindingModel
			{
				Id = id,
				CannedId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["CannedId"].Value),
				Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
				Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value),
				Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
				DateCreate = DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
			};
		}
		private void ComponentsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				_reportLogic.SaveComponentsToWordFile(new ReportBindingModel { FileName = dialog.FileName });
				MessageBox.Show("", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void ComponentCannedToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormReportCannedComponents>();
			if (service is FormReportCannedComponents form)
			{
				form.ShowDialog();
			}
		}

		private void OrdersToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormReportOrders>();
			if (service is FormReportOrders form)
			{
				form.ShowDialog();
			}
		}
		private void ClientsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormClients>();
			if (service is FormClients form)
			{
				form.ShowDialog();
			}
		}
		private void ImplementerToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormImplementers>();
			if (service is FormImplementers form)
			{
				form.ShowDialog();
			}
		}
		private void StartingWorkToolStripMenuItem_Click(object sender, EventArgs e)
		{
			_workProcess.DoWork((DependencyManager.Instance.Resolve<IImplementerLogic>() as IImplementerLogic)!, _orderLogic);
			MessageBox.Show("  ", "",
		   MessageBoxButtons.OK, MessageBoxIcon.Information);
		}
		private void mailToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = DependencyManager.Instance.Resolve<FormViewMail>();
			if (service is FormViewMail form)
			{
				form.ShowDialog();
			}
		}
		private void CreateBackUpToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				if (_backUpLogic != null)
				{
					var fbd = new FolderBrowserDialog();
					if (fbd.ShowDialog() == DialogResult.OK)
					{
						_backUpLogic.CreateBackUp(new BackUpSaveBindingModel
						{
							FolderName = fbd.SelectedPath
						});
						MessageBox.Show(" ", "",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
					}
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "", MessageBoxButtons.OK,
				MessageBoxIcon.Error);
			}
		}
	}
}